const cheerio = require('cheerio');
const puppeteer = require('puppeteer');
let requestPromise = require('request-promise-native');
let moment = require('moment');
moment.locale('pt-br');


async function navigateWebsite(url, idHotel, checkin, checkout, adultos, criancas, browserParam) {
    let browser = null;
    if (browserParam == null){

       browser= await puppeteer.launch({headless: true, args: ['--no-sandbox', '--disable-setuid-sandbox']});
    }else{
        browser = browserParam;
    }

    try {

        const mainTab = await browser.newPage();

        // Document, Stylesheet, Image, Media, Font, Script, TextTrack, XHR, Fetch, EventSource, WebSocket, Manifest, Other.
        await mainTab.setRequestInterception(true);
        mainTab.on('request', request => {
            if (request.resourceType() === 'image' || request.resourceType() === 'font' || request.resourceType() === 'stylesheet') {
                request.abort();
            }
            else {
                request.continue();
            }

        });


        if (criancas && criancas === '0') {
            criancas = '';
        }

        let urlBuild = `${url}/D/Reserva/ConsultaDisponibilidade?checkin=${checkin}&checkout=${checkout}&cidade=&hotel=${idHotel}&adultos=${adultos}&criancas=${criancas}&promocode=&tarifa=`;

        console.log(urlBuild);

        await mainTab.goto(urlBuild, {timeout: 20000});
        await mainTab.waitForSelector('#acomodacoesWrapper', {timeout: 20000});


        const htmlTag = await mainTab.evaluate(function (selector) {
            const selectorHtml = document.querySelector(selector);
            return selectorHtml.innerHTML
        }, 'body'); // returns innerHTML of first matching selector for class "main"

        //    console.log(htmlTag.result.value);

        let $ = cheerio.load(htmlTag);
        let promise = null;

        if ($('#hotel-selecionado-indisponivel-titulo').text() != "") {

            console.log("erro");

            let obj = {
                message: $('#hotel-selecionado-indisponivel-titulo').text()
            };

            promise = await Promise.reject(obj)


        } else {

            let rooms = [];

            $('.row-quarto').each((index, item) => {

                let room = new HotelRoom();

                // console.log( $(item).children().length)

                $(item).children().each((index, td) => {


                    if (index === 0) {

                        let tdx = $(td);

                        room.name = tdx.find('.quartoNome').text();
                        room.mainPhoto = tdx.find('.imgQuarto').attr('src');
                        room.description = tdx.find('.quartoDescricao').find('p').text();
                        room.amountAdults = tdx.find('.adultos').attr('data-n');
                        room.amountKids = tdx.find('.criancas').attr('data-n');

                        if (room.amountKids == 'no') {
                            room.amountKids = 0;
                        }

                        let str = tdx.find('.imgQuarto').attr('onclick');

                        let idStr = str.replace(/[^0-9,]/g, "").split(",");

                        room.id = idStr[1];

                        let modalMoreInfo = $('#modal-container-' + room.id);

                        modalMoreInfo.find('.amenities-list > .amenity-item').each((index, itemAmenite) => {
                            room.amenites.push($(itemAmenite).text());
                        });


                        modalMoreInfo.find('#divGaleria > img').each((index, itemAmenite) => {
                            room.images.push($(itemAmenite).attr('src-full-image'));
                        })


                    } else {


                        let tdx = $(td);

                        room.price = tdx.find('.valorFinal').first().text();
                        room.total = 'R$ ' + tdx.find('.var-cart-valor-total').val() + ",00";
                    }

                    // console.log("td", room)
                });

                let obj = {
                    data: rooms,
                    url: urlBuild,
                };
                rooms.push(room);

                // console.log(rooms);

                promise = Promise.resolve(obj)
            })


        }


        // Close the browser
        if (!browserParam){
             browser.close();
        }



        return promise;

    } catch (err) {
        console.log(err);
        if (!browserParam){
             browser.close();
        }
        throw err;
    }
}


async function getInventory(url, companyId, checkIn, checkOut) {

    try {

        let startDate = moment(checkIn, 'DD/MM/YYYY');
        let endDate = moment(checkOut, 'DD/MM/YYYY');
        let now = moment();

        let diffFromStart = startDate.diff(now, 'days');
        //       console.log(diffFromStart);

        let from = now.format('DD/MM/YYYY');

        if (diffFromStart > 7) {
            from = moment(checkIn, 'DD/MM/YYYY').subtract(7, 'days').format('DD/MM/YYYY');
        }
        //console.log(from);

        let daysOfStay = endDate.diff(startDate, 'days');


        //console.log(daysOfStay);
        let monthsAhead = 3;
        let to = moment(from, 'DD/MM/YYYY').add(monthsAhead, 'months').format('DD/MM/YYYY');

        let response = await requestPromise.get(`${url}/D/Busca/MelhorTarifa?dataInicio=${from}&dataFim=${to}&codCidade=&codHotel=${companyId}`);
        //console.log(response);

        let responseObj = JSON.parse(response).map(r => {
            r.Data = moment(r.Data, 'DD/MM/YYYY');
            return r;
        });

        //console.log(responseObj);

        let possibleSuggestions = [];

        responseObj.forEach(dateObj => {
            let possibleDate = dateObj.Data;
            let closedToArrival = dateObj.CheckinRestrito || false;
            if (((possibleDate.day() === startDate.day()) || isNear(possibleDate, startDate)) && !closedToArrival) {
                possibleSuggestions.push({checkInDate: possibleDate});
            }
        });

        possibleSuggestions.forEach(ps => {
            responseObj.forEach(dateObj => {
                let possibleDate = dateObj.Data;
                let closedToDeparture = dateObj.CheckoutRestrito || false;
                if (possibleDate.diff(ps.checkInDate, 'days') === daysOfStay && !closedToDeparture) {
                    ps.checkOutDate = possibleDate;
                }
            });
        });

        possibleSuggestions = possibleSuggestions.filter(ps => ps.checkOutDate && ps.checkInDate);
        possibleSuggestions = possibleSuggestions.slice(0, 5);

        //console.log(possibleSuggestions);

        return Promise.resolve(possibleSuggestions);

    } catch (err) {
        console.log("caiu aqui no erro");
        console.error(err);
        return Promise.reject(err);
    }

}

isNear = function (d1, d2) {
    return Math.abs(d1.diff(d2, 'days')) <= 1;
};


getSuggestionDates = async function (url, companyId, checkIn, checkOut, AmountAdults, AmountChildren) {
    let browser = await puppeteer.launch({headless: true, args: ['--no-sandbox', '--disable-setuid-sandbox']});
    try {
        let possibleDates = await getInventory(url, companyId, checkIn, checkOut);

        let suggestions = [];
        for (let pd of possibleDates) {
            if (suggestions.length < 4) {
                let checkInDate = pd.checkInDate.format('DD/MM/YYYY');
                let checkOutDate = pd.checkOutDate.format('DD/MM/YYYY');
                try {


                    let availability = await navigateWebsite(url, companyId, checkInDate, checkOutDate, AmountAdults, AmountChildren, browser);

                    if (availability.data && availability.data.length > 0) {
                        pd.minimumPrice = availability.data[0].price;
                        suggestions.push(pd);
                    }
                } catch (err) {
                    console.error(err);
                }
            }
        }


        suggestions = suggestions.map(s => {
            let suggestion = {};
            suggestion.checkIn = s.checkInDate.format('DD/MM/YYYY');
            suggestion.checkOut = s.checkOutDate.format('DD/MM/YYYY');
            let desiredCheckIn = moment(checkIn, 'DD/MM/YYYY');
            suggestion.text = getSuggestionText(s.checkInDate, desiredCheckIn);
            suggestion.period = (s.checkInDate.day() === desiredCheckIn.day() ? 'Mesmos dias da semana' : 'Dias próximos');
            suggestion.minimumPrice = s.minimumPrice;

            return suggestion;
        });

        console.log(suggestions);

        if (suggestions && suggestions.length > 0) {
            return Promise.resolve(suggestions);
        } else {
            return Promise.reject(null);
        }

    } catch (err) {
        console.error(err);
        return Promise.reject(err);
    }finally {
        browser.close();
    }
};


getSuggestionText = function (d1, d2) {
    let text = {};
    let fromTxt = d1.from(d2, true);
    text.amount = fromTxt.split(" ")[0];
    text.param = fromTxt.split(" ")[1];

    if (d1.isBefore(d2)) {
        text.param += " antes";
    } else {
        text.param += " depois";
    }
    return text;
};


// navigateWebsite('https://gjphotels.letsbook.com.br', 26, '14/05/2018', '15/05/2018', 2, 0).then(data => {
//
//     console.log(data.data.length);
//     console.log("sucesso")
// }).catch(error => {
//     console.log("caiu aqui");
//     console.log(error.message);
// });


// getSuggestionDates('https://pratasthermas.letsbook.com.br', 12, '23/03/2018', '25/03/2018', 2, 1).then(data => {
//     console.log("sucesso")
// }).catch(error => {
//     console.log("caiu aqui");
//     console.log(error.message);
// });


// getInventory('https://gjphotels.letsbook.com.br', 26, '14/05/2018', '15/05/2018').then(data => {
//      console.log("sucesso")
//  }).catch(error => {
//      console.log("caiu aqui");
//      console.log(error.message);
// });


class HotelRoom {

    constructor() {
        this.amenites = new Array();
        this.images = new Array();
    }

}


module.exports = {search: navigateWebsite, getSuggestionDates: getSuggestionDates};



