const cheerio = require('cheerio');
const puppeteer = require('puppeteer');
const applyEvasions = require('./apply-evasions');


async function navigateWebsite(idHotel, checkin, checkout, adultos, criancas) {
    const browser = await puppeteer.launch({headless: true, args: ['--no-sandbox', '--disable-setuid-sandbox']});
    try {

        const mainTab = await browser.newPage();

        await applyEvasions(mainTab);

        // Document, Stylesheet, Image, Media, Font, Script, TextTrack, XHR, Fetch, EventSource, WebSocket, Manifest, Other.
        await mainTab.setRequestInterception(true);
        mainTab.on('request', request => {
            if (request.resourceType() === 'image' || request.resourceType() === 'font' || request.resourceType() === 'stylesheet' ) {
                request.abort();
            }
            else {
                request.continue();
            }

        });
        // Navigate to a URL

        if (criancas && criancas === '0') {
            criancas = '';
        }

        var baseURL = "https://reservasonlinebrasil.com.br/"
        var url = baseURL + "v3/motor3.asp?"


        // https://reservasonlinebrasil.com.br/v3/motor3.asp?diachegada=07/03/2018&diasaida=09/03/2018&estabelecimento=0019&acomodacao=2&adt=2&chd5=0&chd12=0&valor=&ano=&valor1=&ano1=&origem=Site_Estabelecimento&passo=2&identificador=201836114058001920421&cupom=

        let identificador = 2018 + "" + new Date().getTime();
        let urlBuild = `${url}diachegada=${checkin}&diasaida=${checkout}&estabelecimento=${idHotel}&adt=${adultos}&chd5=${criancas}&passo=2&identificador=${identificador}`;

        console.log(urlBuild);

        await mainTab.goto(urlBuild, {timeout: 20000,  waitUntil: 'load'})
        await mainTab.waitForSelector('.acomodacoes', {timeout: 20000})


        const htmlTag = await mainTab.evaluate(function (selector) {
            const selectorHtml = document.querySelector(selector)
            return selectorHtml.innerHTML
        }, '#main'); // returns innerHTML of first matching selector for class "main"


        // await  mainTab.wait(2000000)

        let $ = cheerio.load(htmlTag);


        let promise = null;

        if ($('.aviso').text() != "") {

            console.log($('.aviso').text().trim());

            let obj = {
                message: $('.aviso').text().trim()
            };

            promise = await Promise.reject(obj)

        } else {

            let rooms = [];

            $('.acomodacao').each((index, item) => {


                console.log("entrei aqui")

                let room = new HotelRoom();

                // console.log( $(item).children().length)

                // $(item).children().each((index, td) => {


                let tdx = $(item);

                room.name = tdx.find('[itemprop="name"]').text()

                room.description = tdx.find('[itemprop="description"]').text()

                room.price = tdx.find(".media").text().split("x")[0].replace("Diaria:", "")
                room.total = tdx.find('.valor').text()
                room.amountAdults = tdx.find('.tcol > .icone > .fa-user').length;
                //
                tdx.find('.icones  > .icone > i').each((index, itemAmenite) => {
                    room.amenites.push($(itemAmenite).attr("title"));
                })


                tdx.find('[itemprop="image"]').each((index, itemAmenite) => {
                    let img = $(itemAmenite).attr('href');
                    let replacer = img.replace("../", "")

                    room.images.push(baseURL + replacer);
                })

                room.mainPhoto = room.images[0];
                rooms.push(room)
            })

            let obj = {
                data: rooms,
                url: urlBuild,
            };


            console.log(rooms);

            promise = Promise.resolve(obj)
            // })


        }


        // Close the browser
        await browser.close()


        return promise;

    } catch (err) {
        console.log(err);
        browser.close()
        throw err;
    }
}

//
// navigateWebsite('0040', '23/04/2018', '26/04/2018', 2, 1).then(data => {
//     console.log("sucesso")
// }).catch(error => {
//     console.log("caiu aqui")
//     console.log(error.message);
// })

class HotelRoom {

    constructor() {
        this.amenites = new Array();
        this.images = new Array();
    }

}


module.exports = {search: navigateWebsite}


